@@ -31,10 +31,15 @@
 
 #define CUPS_DATADIR    "/usr/share/cups"
 
-KCupsRequest::KCupsRequest() :
+KCupsRequest::KCupsRequest(KCupsConnection *connection) :
+    m_connection(connection),
     m_finished(true),
     m_error(IPP_OK)
 {
+    // If no connection was specified use default one
+    if (m_connection == 0) {
+        m_connection = KCupsConnection::global();
+    }
     connect(this, SIGNAL(finished()), &m_loop, SLOT(quit()));
 }
 
@@ -53,18 +58,18 @@ QString KCupsRequest::serverError() const
 
 void KCupsRequest::getPPDS(const QString &make)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         QVariantHash request;
         if (!make.isEmpty()){
             request["ppd-make-and-model"] = make;
         }
         request["need-dest-name"] = false;
 
-        m_ppds = KCupsConnection::request(CUPS_GET_PPDS,
-                                          "/",
-                                          request,
-                                          true);
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        m_ppds = m_connection->request(CUPS_GET_PPDS,
+                                       "/",
+                                       request,
+                                       true);
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getPPDS", make);
@@ -96,20 +101,39 @@ static void choose_device_cb(const char *device_class,           /* I - Class */
 
 void KCupsRequest::getDevices(int timeout)
 {
-    if (KCupsConnection::readyToStart()) {
+    getDevices(timeout, QStringList(), QStringList());
+}
+
+void KCupsRequest::getDevices(int timeout, QStringList includeSchemes, QStringList excludeSchemes)
+{
+    if (m_connection->readyToStart()) {
         do {
+            const char *include;
+            if (includeSchemes.isEmpty()) {
+                include = CUPS_INCLUDE_ALL;
+            } else {
+                include = includeSchemes.join(QLatin1String(",")).toUtf8();
+            }
+
+            const char *exclude;
+            if (excludeSchemes.isEmpty()) {
+                exclude = CUPS_EXCLUDE_NONE;
+            } else {
+                exclude = excludeSchemes.join(QLatin1String(",")).toUtf8();
+            }
+
             // Scan for devices for "timeout" seconds
             cupsGetDevices(CUPS_HTTP_DEFAULT,
                            timeout,
-                           CUPS_INCLUDE_ALL,
-                           CUPS_EXCLUDE_NONE,
+                           include,
+                           exclude,
                            (cups_device_cb_t) choose_device_cb,
                            this);
-        } while (KCupsConnection::retry("/admin/"));
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        } while (m_connection->retry("/admin/"));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished(true);
     } else {
-        invokeMethod("getDevices", timeout);
+        invokeMethod("getDevices", timeout, includeSchemes, excludeSchemes);
     }
 }
 
@@ -126,23 +150,23 @@ void KCupsRequest::getPrinters(QStringList attributes, cups_ptype_t mask)
 
 void KCupsRequest::getPrinters(QStringList attributes, const QVariantHash &arguments)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         QVariantHash request = arguments;
         request[KCUPS_PRINTER_TYPE] = CUPS_PRINTER_LOCAL;
         request["requested-attributes"] = attributes;
         request["need-dest-name"] = true;
 
         ReturnArguments ret;
-        ret = KCupsConnection::request(CUPS_GET_PRINTERS,
-                                       "/",
-                                       request,
-                                       true);
+        ret = m_connection->request(CUPS_GET_PRINTERS,
+                                    "/",
+                                    request,
+                                    true);
 
         foreach (const QVariantHash &arguments, ret) {
             m_printers << KCupsPrinter(arguments);
         }
 
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getPrinters", qVariantFromValue(attributes), arguments);
@@ -151,7 +175,7 @@ void KCupsRequest::getPrinters(QStringList attributes, const QVariantHash &argum
 
 void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass, QStringList attributes)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         QVariantHash request;
         request[KCUPS_PRINTER_NAME] = printerName;
         request["printer-is-class"] = isClass;
@@ -159,10 +183,10 @@ void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass
         request["requested-attributes"] = attributes;
 
         ReturnArguments ret;
-        ret = KCupsConnection::request(IPP_GET_PRINTER_ATTRIBUTES,
-                                       "/admin/",
-                                       request,
-                                       true);
+        ret = m_connection->request(IPP_GET_PRINTER_ATTRIBUTES,
+                                    "/admin/",
+                                    request,
+                                    true);
 
         foreach (const QVariantHash &arguments, ret) {
             // Inject the printer name back to the arguments hash
@@ -171,7 +195,7 @@ void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass
             m_printers << KCupsPrinter(args);
         }
 
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getPrinterAttributes", printerName, isClass, qVariantFromValue(attributes));
@@ -180,7 +204,7 @@ void KCupsRequest::getPrinterAttributes(const QString &printerName, bool isClass
 
 void KCupsRequest::getJobs(const QString &printerName, bool myJobs, int whichJobs, QStringList attributes)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         QVariantHash request;
         // printer-uri makes the Name of the Job and owner came blank lol
         request[KCUPS_PRINTER_NAME] = printerName;
@@ -201,15 +225,15 @@ void KCupsRequest::getJobs(const QString &printerName, bool myJobs, int whichJob
         request["group-tag-qt"] = IPP_TAG_JOB;
 
         ReturnArguments ret;
-        ret = KCupsConnection::request(IPP_GET_JOBS,
-                                       "/",
-                                       request,
-                                       true);
+        ret = m_connection->request(IPP_GET_JOBS,
+                                    "/",
+                                    request,
+                                    true);
         foreach (const QVariantHash &arguments, ret) {
             m_jobs << KCupsJob(arguments);
         }
 
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getJobs", printerName, myJobs, whichJobs, qVariantFromValue(attributes));
@@ -218,7 +242,7 @@ void KCupsRequest::getJobs(const QString &printerName, bool myJobs, int whichJob
 
 void KCupsRequest::getJobAttributes(int jobId, const QString &printerUri, QStringList attributes)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         QVariantHash request;
         request[KCUPS_JOB_ID] = jobId;
         request[KCUPS_PRINTER_URI] = printerUri;
@@ -226,79 +250,44 @@ void KCupsRequest::getJobAttributes(int jobId, const QString &printerUri, QStrin
         request["requested-attributes"] = attributes;
 
         ReturnArguments ret;
-        ret = KCupsConnection::request(IPP_GET_JOB_ATTRIBUTES,
-                                       "/admin/",
-                                       request,
-                                       true);
+        ret = m_connection->request(IPP_GET_JOB_ATTRIBUTES,
+                                    "/admin/",
+                                    request,
+                                    true);
 
         foreach (const QVariantHash &arguments, ret) {
             m_jobs << KCupsJob(arguments);
         }
 
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getJobAttributes", jobId, printerUri, qVariantFromValue(attributes));
     }
 }
 
-void KCupsRequest::createDBusSubscription(const QStringList &events)
-{
-    if (KCupsConnection::readyToStart()) {
-        int ret;
-        ret = KCupsConnection::global()->createDBusSubscription(events);
-        kDebug() << "Got internal ID" << ret << events;
-        m_subscriptionId = ret;
-
-        if (ret < 0) {
-            setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
-        }
-        setFinished();
-    } else {
-        invokeMethod("createDBusSubscription", events);
-    }
-}
-
-void KCupsRequest::cancelDBusSubscription(int subscriptionId)
-{
-    if (KCupsConnection::readyToStart()) {
-        KCupsConnection::global()->removeDBusSubscription(subscriptionId);
-
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
-        setFinished();
-    } else {
-        invokeMethod("cancelDBusSubscription", subscriptionId);
-    }
-}
-
-//void KCupsRequest::addClass(const QVariantHash &values)
-//{
-//    QVariantHash request = values;
-//    request["printer-is-class"] = true;
-//    request["printer-is-accepting-jobs"] = true;
-//    request[KCUPS_PRINTER_STATE] = IPP_PRINTER_IDLE;
-
-//    doOperation(CUPS_ADD_MODIFY_CLASS, QLatin1String("/admin/"), request);
-//}
-
 void KCupsRequest::getServerSettings()
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         do {
             int num_settings;
             cups_option_t *settings;
             QVariantHash arguments;
-            cupsAdminGetServerSettings(CUPS_HTTP_DEFAULT, &num_settings, &settings);
+            int ret = cupsAdminGetServerSettings(CUPS_HTTP_DEFAULT, &num_settings, &settings);
             for (int i = 0; i < num_settings; ++i) {
                 QString name = QString::fromUtf8(settings[i].name);
                 QString value = QString::fromUtf8(settings[i].value);
                 arguments[name] = value;
             }
             cupsFreeOptions(num_settings, settings);
+            if (ret) {
+                setError(HTTP_OK, IPP_OK, QString());
+            } else {
+                setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
+            }
 
             m_server = KCupsServer(arguments);
-        } while (KCupsConnection::retry("/admin/"));
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        } while (m_connection->retry("/admin/"));
         setFinished();
     } else {
         invokeMethod("getServerSettings");
@@ -307,15 +296,15 @@ void KCupsRequest::getServerSettings()
 
 void KCupsRequest::getPrinterPPD(const QString &printerName)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         do {
             const char  *filename;
             filename = cupsGetPPD2(CUPS_HTTP_DEFAULT, printerName.toUtf8());
             kDebug() << filename;
             m_ppdFile = filename;
             kDebug() << m_ppdFile;
-        } while (KCupsConnection::retry("/"));
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        } while (m_connection->retry("/"));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("getPrinterPPD", printerName);
@@ -324,7 +313,7 @@ void KCupsRequest::getPrinterPPD(const QString &printerName)
 
 void KCupsRequest::setServerSettings(const KCupsServer &server)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         do {
             QVariantHash args = server.arguments();
             int num_settings = 0;
@@ -341,8 +330,8 @@ void KCupsRequest::setServerSettings(const KCupsServer &server)
 
             cupsAdminSetServerSettings(CUPS_HTTP_DEFAULT, num_settings, settings);
             cupsFreeOptions(num_settings, settings);
-        } while (KCupsConnection::retry("/admin/"));
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        } while (m_connection->retry("/admin/"));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("setServerSettings", qVariantFromValue(server));
@@ -460,7 +449,7 @@ void KCupsRequest::printTestPage(const QString &printerName, bool isClass)
 
 void KCupsRequest::printCommand(const QString &printerName, const QString &command, const QString &title)
 {
-    if (KCupsConnection::readyToStart()) {
+    if (m_connection->readyToStart()) {
         do {
             int           job_id;                 /* Command file job */
             char          command_file[1024];     /* Command "file" */
@@ -485,7 +474,7 @@ void KCupsRequest::printCommand(const QString &printerName, const QString &comma
                                         &hold_option)) < 1) {
                 qWarning() << "Unable to send command to printer driver!";
 
-                setError(IPP_NOT_POSSIBLE, i18n("Unable to send command to printer driver!"));
+                setError(HTTP_OK, IPP_NOT_POSSIBLE, i18n("Unable to send command to printer driver!"));
                 setFinished();
                 return;
             }
@@ -498,23 +487,23 @@ void KCupsRequest::printCommand(const QString &printerName, const QString &comma
                                        1);
             if (status == HTTP_CONTINUE) {
                 status = cupsWriteRequestData(CUPS_HTTP_DEFAULT, command_file,
-                                            strlen(command_file));
+                                              strlen(command_file));
             }
 
             if (status == HTTP_CONTINUE) {
                 cupsFinishDocument(CUPS_HTTP_DEFAULT, printerName.toUtf8());
             }
 
-            setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
-            if (KCupsConnection::lastError() >= IPP_REDIRECTION_OTHER_SITE) {
+            setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
+            if (httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError() >= IPP_REDIRECTION_OTHER_SITE) {
                 qWarning() << "Unable to send command to printer driver!";
 
                 cupsCancelJob(printerName.toUtf8(), job_id);
                 setFinished();
                 return; // Return to avoid a new try
             }
-        } while (KCupsConnection::retry("/"));
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        } while (m_connection->retry("/"));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("printCommand", printerName, command, title);
@@ -589,12 +578,11 @@ void KCupsRequest::invokeMethod(const char *method,
     m_jobs.clear();
     m_ppds.clear();
     m_ppdFile.clear();
-    m_subscriptionId = -1;
 
     // If this fails we get into a infinite loop
     // Do not use global()->thread() which point
     // to the KCupsConnection parent thread
-    moveToThread(KCupsConnection::global());
+    moveToThread(m_connection);
 
     m_finished = !QMetaObject::invokeMethod(this,
                                             method,
@@ -608,20 +596,20 @@ void KCupsRequest::invokeMethod(const char *method,
                                             QGenericArgument(arg7.typeName(), arg7.data()),
                                             QGenericArgument(arg8.typeName(), arg8.data()));
     if (m_finished) {
-        setError(IPP_BAD_REQUEST, i18n("Failed to invoke method: %1", method));
+        setError(HTTP_ERROR, IPP_BAD_REQUEST, i18n("Failed to invoke method: %1", method));
         setFinished();
     }
 }
 
 void KCupsRequest::doOperation(int operation, const QString &resource, const QVariantHash &request)
 {
-    if (KCupsConnection::readyToStart()) {
-        KCupsConnection::request(static_cast<ipp_op_e>(operation),
-                                 resource.toUtf8(),
-                                 request,
-                                 false);
+    if (m_connection->readyToStart()) {
+        m_connection->request(static_cast<ipp_op_e>(operation),
+                              resource.toUtf8(),
+                              request,
+                              false);
 
-        setError(KCupsConnection::lastError(), QString::fromUtf8(cupsLastErrorString()));
+        setError(httpGetStatus(CUPS_HTTP_DEFAULT), cupsLastError(), QString::fromUtf8(cupsLastErrorString()));
         setFinished();
     } else {
         invokeMethod("doOperation", operation, resource, request);
@@ -643,11 +631,6 @@ QString KCupsRequest::printerPPD() const
     return m_ppdFile;
 }
 
-int KCupsRequest::subscriptionId() const
-{
-    return m_subscriptionId;
-}
-
 KCupsPrinters KCupsRequest::printers() const
 {
     return m_printers;
@@ -677,13 +660,24 @@ ipp_status_t KCupsRequest::error() const
     return m_error;
 }
 
+http_status_t KCupsRequest::httpStatus() const
+{
+    return m_httpStatus;
+}
+
 QString KCupsRequest::errorMsg() const
 {
     return m_errorMsg;
 }
 
-void KCupsRequest::setError(ipp_status_t error, const QString &errorMsg)
+KCupsConnection *KCupsRequest::connection() const
 {
+    return m_connection;
+}
+
+void KCupsRequest::setError(http_status_t httpStatus, ipp_status_t error, const QString &errorMsg)
+{
+    m_httpStatus = httpStatus;
     m_error = error;
     m_errorMsg = errorMsg;
 }
